@@ -1,8 +1,10 @@
+import sys
 import os
 import time
 import pytz
 import base64
 import boto3
+import botocore
 import logging
 import subprocess
 import json
@@ -15,6 +17,15 @@ from botocore.exceptions import WaiterError
 logging.basicConfig(level=logging.INFO)
 
 
+# exception hook so errors raise to main thread
+def exception_hook(args):
+    raise args.exc_type(args.exc_value)
+    sys.exit(1)
+
+# set the exception hook
+threading.excepthook = exception_hook
+
+
 class EC2Manager:
     def __init__(self, **kwargs):
         # needed envs
@@ -52,10 +63,14 @@ class EC2Manager:
             self._instance_names = self._get_instance_names()
             self._instance_repo_updates = self._get_instance_repo_updates()
 
+            # boost the pool size of urllib to handle concurrent for each instance
+            client_config = botocore.config.Config(
+                max_pool_connections=len(self._instance_names),
+            )
             self._aws_session = boto3.Session(region_name=self._aws_default_region)
-            self._ssm_client = self._aws_session.client('ssm')
-            self._ec2_client = self._aws_session.client('ec2')
-            self._s3_client = self._aws_session.client('s3')
+            self._ssm_client = self._aws_session.client('ssm', config=client_config)
+            self._ec2_client = self._aws_session.client('ec2', config=client_config)
+            self._s3_client = self._aws_session.client('s3', config=client_config)
 
             account_number = self._aws_session.client('sts').get_caller_identity().get('Account')
             if not account_number:
